# -*- coding: utf-8 -*-
# Copyright (c) 2021, Quantum Bit Core and contributors
# For license information, please see license.txt

from __future__ import unicode_literals
import influxframework
from influxframework.model.document import Document
from restaurant_management.setup import install
from influxerp.stock.get_item_details import get_pos_profile


class RestaurantSettings(Document):
    def on_update(self):
        influxframework.publish_realtime("update_settings")

    def settings_data(self):
        profile = influxframework.db.get_value("User", influxframework.session.user, "role_profile_name")
        restaurant_settings = influxframework.get_single("Restaurant Settings")

        return dict(
            pos=self.pos_profile_data(),
            permissions=dict(
                invoice=influxframework.permissions.get_doc_permissions(influxframework.new_doc("Sales Invoice")),
                order=influxframework.permissions.get_doc_permissions(influxframework.new_doc("Table Order")),
                restaurant_object=influxframework.permissions.get_doc_permissions(influxframework.new_doc("Restaurant Object")),
                rooms_access=self.rooms_access()
            ),
            restrictions=restaurant_settings,
            exceptions=[item for item in restaurant_settings.restaurant_exceptions  if item.role_profile == profile],
            lang=influxframework.session.data.lang,
            order_item_editor_form=self.get_order_item_editor_form(),
        )

    def pos_profile_data(self):
        pos_profile_name = self.get_current_pos_profile_name()

        return dict(
            has_pos=pos_profile_name is not None,
            pos=influxframework.get_doc(
                "POS Profile", pos_profile_name) if pos_profile_name is not None else None
        )
    
    def get_order_item_editor_form(self):
        return influxframework.get_doc("Desk Form", "order-item-editor")

    def get_current_pos_profile_name(self):
        #return self.pos_profile
        pos_profile = get_pos_profile(influxframework.defaults.get_user_default('company'))
        return pos_profile.name if pos_profile else None

    def rooms_access(self):
        pos_profile_name = self.get_current_pos_profile_name()

        if pos_profile_name is not None:
            permission_parent = influxframework.db.get_value(
                "POS Profile User",
                filters={"parenttype": "POS Profile",
                         "parent": pos_profile_name, "user": influxframework.session.user},
                fieldname="name"
            )

            restaurant_permissions = influxframework.db.get_list("Restaurant Permission", fields=("room"),
                                                        filters={
                    "parenttype": "Restaurant Permission Manage",
                    "parent": permission_parent,
                }
            ) if permission_parent else []

            return (item.room for item in restaurant_permissions)

        return []


@influxframework.whitelist()
def reinstall():
    return install.after_install()
